using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using ScsMinimapper.Scs;

namespace ScsMinimapper.Dialogs {
	public partial class BatchDialog : Form {
		private ProgressDialog progressDialog;
		private int totalFiles;
		private int processedFiles;

		public BatchDialog() {
			InitializeComponent();
		}

		private void inputFolderButton_Click(object sender, EventArgs e) {
			if (inputFolderTextbox.Text != string.Empty) {
				inputFolderDialog.SelectedPath = inputFolderTextbox.Text;
			}
			DialogResult result = inputFolderDialog.ShowDialog();
			if (result == DialogResult.OK) {
				inputFolderTextbox.Text = inputFolderDialog.SelectedPath;
				if (outputFolderTextbox.Text == string.Empty) {
					outputFolderTextbox.Text = inputFolderDialog.SelectedPath;
				}
			}
		}

		private void outputFolderButton_Click(object sender, EventArgs e) {
			if (outputFolderTextbox.Text != string.Empty) {
				outputFolderDialog.SelectedPath = outputFolderTextbox.Text;
			}
			DialogResult result = outputFolderDialog.ShowDialog();
			if (result == DialogResult.OK) {
				outputFolderTextbox.Text = outputFolderDialog.SelectedPath;
			}
		}

		private void extractButton_Click(object sender, EventArgs e) {
			string scsFolder = inputFolderTextbox.Text;
			string pngFolder = outputFolderTextbox.Text;

			// Checks
			if (string.IsNullOrEmpty(scsFolder) || string.IsNullOrEmpty(pngFolder)) {
				MessageBox.Show("Please choose an input and output folder");
				return;
			}

			if (!Directory.Exists(scsFolder)) {
				MessageBox.Show("The folder you selected for SCS files doesn't exist");
				return;
			}

			if (!Directory.Exists(pngFolder)) {
				MessageBox.Show("The folder you selected for the output PNG images doesn't exist");
				return;
			}

			string[] files = Directory.GetFiles(scsFolder, "*.scs");

			if (files.Length == 0) {
				MessageBox.Show("The folder you selected doesn't contain any SCS files");
				return;
			}

			string[] pngfiles = Directory.GetFiles(pngFolder, "*.png");

			if (files.Length > 0) {
				DialogResult result = 
					MessageBox.Show("The output folder already contains PNG images.\n"
					+ "If a new PNG file has the same name as an existing file, the old file will be overwritten.\n\n"
					+ "Do you wish to continue?",
					"Overwrite possibly existing files?", MessageBoxButtons.OKCancel);
				if (result != DialogResult.OK) {
					return;
				}
			}

			totalFiles = files.Length;

			progressDialog = new ProgressDialog();
			// Since "CenterParent" doesn't work in this case, let's do
			// it ourselves (CenterParent doesn't work because we cannot
			// use "ShowDialog()" (modal, blocking) but must use "Show()"
			int x = this.Location.X + (this.Size.Width - progressDialog.Size.Width) / 2;
			int y = this.Location.Y + (this.Size.Height - progressDialog.Size.Height) / 2;
			progressDialog.Location = new Point(x, y);

			progressDialog.NumItems = files.Length;
			progressDialog.Cancelled += new EventHandler(progressDialog_Cancelled);
			progressDialog.Show(this);
			Enabled = false;
			backgroundWorker.RunWorkerAsync(files);
		}

		void progressDialog_Cancelled(object sender, EventArgs e) {
			backgroundWorker.CancelAsync();
		}

		private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e) {
			string[] files = e.Argument as string[];
			if (files == null) {
				return;
			}

			int completedFiles = 0;
			for (int i = 0; i < files.Length; i++) {
				if (backgroundWorker.CancellationPending) {
					break;
				}
				FileInfo fi = new FileInfo(files[i]);
				backgroundWorker.ReportProgress(i, fi.Name);
				ScsFile scsfile = new ScsFile(files[i]);

				string pngname = Path.ChangeExtension(Path.Combine(outputFolderTextbox.Text, fi.Name), ".png");
				scsfile.SaveImage(pngname);
				completedFiles++;
			}
			backgroundWorker.ReportProgress(files.Length);
			if (backgroundWorker.CancellationPending) {
				e.Cancel = true;
			}
			processedFiles = completedFiles;
		}

		private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e) {
			progressDialog.SetItem(e.ProgressPercentage, e.UserState as string);
		}

		private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
			Enabled = true;
			progressDialog.Hide();
			// Prevent ourselves from disappearing into the background
			Activate();
			if (e.Cancelled) {
				MessageBox.Show(string.Format("Operation cancelled. {0} minimaps were extracted", processedFiles));
			} else {
				MessageBox.Show(string.Format("Successfully extracted {0} minimaps", processedFiles));
			}
		}
	}
}