using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;

namespace ScsMinimapper.Scs {
	/// <summary>
	/// Class for decompressing SCS files
	/// </summary>
	class Decompressor {
		private string filename;

		/// <summary>
		/// Creates a new decompressor for the specified file
		/// </summary>
		/// <param name="filename">File to decompress</param>
		public Decompressor(string filename) {
			this.filename = filename;
		}

		/// <summary>
		/// Decompresses the file and returns its contents as byte array
		/// </summary>
		/// <returns>Decompressed contents, or NULL if an error occurred
		/// during decompression</returns>
		public byte[] Decompress() {
			try {
				FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read);
				try {
					GZipStream gzip = new GZipStream(fs, CompressionMode.Decompress);
					return GetByteArray(gzip);
				} catch {
					fs.Seek(0, SeekOrigin.Begin);
					if (FindPattern(fs, new byte[] { (byte)'Z', (byte)'L', (byte)'I', (byte)'B' })) {
						int length = (int)ReadInt(fs);
						fs.Seek(6, SeekOrigin.Current); // null int plus 2 deflate header bytes
						DeflateStream deflate = new DeflateStream(fs, CompressionMode.Decompress);
						return GetByteArray(deflate, length);
					}
				} finally {
					fs.Close();
				}
			} catch { }
			return null;
		}

		/// <summary>
		/// Reads data from a Gzip stream and returns it as byte array
		/// </summary>
		/// <param name="stream">Stream to read from</param>
		/// <returns>Data read from the stream</returns>
		/// <exception cref="IOException"></exception>
		private byte[] GetByteArray(GZipStream stream) {
			int bufsize = 1048576; // 1 MB
			List<byte[]> chunks = new List<byte[]>();
			int readbytes, totalbytes = 0;

			// Read the file by chunks of one meg until we reach the end
			// (stream.Length is not supported for gzip streams)
			do {
				byte[] buffer = new byte[bufsize];
				readbytes = stream.Read(buffer, 0, bufsize);
				totalbytes += readbytes;
				chunks.Add(buffer);
			} while (readbytes == bufsize);

			// Combine buffers into one big buffer
			byte[] final = new byte[totalbytes];
			int start = 0;
			foreach (byte[] buffer in chunks) {
				Array.Copy(buffer, 0, final, start, Math.Min(buffer.Length, totalbytes - start));
				start += buffer.Length;
			}
			return final;
		}

		/// <summary>
		/// Reads data from a Deflate stream and returns it
		/// </summary>
		/// <param name="stream">Stream to read from</param>
		/// <param name="length">Length of the uncompressed data</param>
		/// <returns>Data read from the stream</returns>
		/// <exception cref="IOException"></exception>
		private byte[] GetByteArray(DeflateStream stream, int length) {
			byte[] buffer = new byte[length];
			int read = stream.Read(buffer, 0, length);
			if (read != length) {
				return null;
			}
			return buffer;
		}

		/// <summary>
		/// Finds a pattern in the stream
		/// </summary>
		/// <param name="fs">Stream to search</param>
		/// <param name="pattern">Pattern to search for</param>
		/// <returns>True if the pattern was found, false otherwise</returns>
		private bool FindPattern(Stream fs, byte[] pattern) {
			bool found = false;
			bool ok = true;
			while (ok && !found) {
				int result = fs.ReadByte();
				if (result == pattern[0]) {
					// Try to match the rest
					found = true;
					for (int i = 1; i < pattern.Length; i++) {
						if (fs.ReadByte() != pattern[i]) {
							found = false;
							break;
						}
					}
				} else if (result == -1) {
					ok = false;
				}
			}
			return found;
		}

		/// <summary>
		/// Reads an unsigned, little endian integer from the stream
		/// </summary>
		/// <param name="stream">Stream to read from</param>
		/// <returns>Integer read, or 0 if EOF reached</returns>
		private uint ReadInt(Stream stream) {
			byte[] buf = new byte[4];
			if (4 != stream.Read(buf, 0, 4)) {
				return 0;
			}
			return (uint)((buf[3] << 24) | (buf[2] << 16) | (buf[1] << 8) | (buf[0]));
		}
	}
}
