using System.Drawing;
using System.Drawing.Imaging;

namespace ScsMinimapper.Scs.Readers {
	/// <summary>
	/// Class for reading the roads from a scs file
	/// </summary>
	class RoadReader {
		private Input input;
		private int size;
		private static Color ROAD_COLOR = Color.FromArgb(110, 110, 110);

		/// <summary>
		/// Constructor - creates a new road reader with the specified
		/// input. The input should be set just *after* the last byte
		/// of the road section.
		/// </summary>
		/// <param name="input">SCS input file</param>
		/// <param name="size">Map size</param>
		public RoadReader(Input input, int size) {
			this.input = input;
			this.size = size;
		}

		/// <summary>
		/// Gets the roads from the input and returns it as an image
		/// </summary>
		/// <returns>RGBA image with roads painted on it</returns>
		public Bitmap GetRoads() {
			// Assume we're just after the last byte of the roads
			RasterImage image = new RasterImage(size, size, PixelFormat.Format32bppArgb);
			bool ok = true;

			while (ok) {
				// Check whether it's a 4x1 or 4x4 file
				int roadLength = 0;
				input.SkipBytes(-4);
				uint check = input.ReadInt();
				if (check == 1) {
					// seems like a road!
					input.SkipBytes(-(4 * 16 + 4));
					check = input.ReadInt();
					if (check == 4) {
						// 4x1 road
						roadLength = 1;
					} else {
						// Check for 4x4 road segment
						// We sought back 4 road tiles, should go 12 more
						input.SkipBytes(-(12 * 16 + 4));
						check = input.ReadInt();
						if (check == 16) {
							roadLength = 4;
						} else {
							ok = false;
						}
					}
				} else {
					ok = false;
				}
				if (ok)
					ok = PlaceRoad(image, roadLength * 4);

				// Seek back
				input.SkipBytes(-4 * (roadLength * 16 + 2));
			}

			// Find the other type of road:
			byte[] pattern = { 0xec, 0x1b, 0x2a, 0x32 };
			if (input.FindPattern(pattern)) {
				// Seek back 16 bytes
				input.SkipBytes(-16);
				int amount = (int)input.ReadInt();
				PlaceRoadv2(image, amount);
			}
			return image.ToBitmap();
		}

		/// <summary>
		/// Place a piece of road, either 4 tiles or 16 tiles (4x4)
		/// </summary>
		/// <param name="image">Image to paint the road on</param>
		/// <param name="amount">Number of tiles to paint, 4 or 16</param>
		/// <returns>True if the full road was placed successfully,
		/// false otherwise</returns>
		private bool PlaceRoad(RasterImage image, int amount) {
			for (int i = 0; i < amount; i++) {
				int x = 1 + (int)input.ReadInt();
				int y = (int)input.ReadInt();
				if (x > image.Width || y > image.Height) {
					return false;
				}
				image.SetPixel(x, y, ROAD_COLOR);
				input.SkipBytes(4);
				uint check = input.ReadInt();
				if (check != 1) {
					return false;
				}
			}
			return true;
		}

		/// <summary>
		/// Place non-highway roads.
		/// </summary>
		/// <param name="image">Image to paint the road on</param>
		/// <param name="amount">Number of road tiles to paint</param>
		private void PlaceRoadv2(RasterImage image, int amount) {
			for (int i = 0; i < amount; i++) {
				int x = 1 + (int)input.ReadInt();
				int y = (int)input.ReadInt();
				if (x > image.Width || y > image.Height) {
					return;
				}
				image.SetPixel(x, y, ROAD_COLOR);
				input.SkipBytes(8);
			}
		}
	}
}
