using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using ScsMinimapper.ImageFormats;
using ScsMinimapper.Scs.Readers;

namespace ScsMinimapper.Scs {
	class ScsFile {
		private string filename;
		private string errorMessage;
		private Bitmap image;

		/// <summary>
		/// Error message when loading the file failed.
		/// </summary>
		public string ErrorMessage {
			get { return errorMessage; }
		}

		/// <summary>
		/// Gets the minimap image. Null on failure.
		/// Must have called Process() before.
		/// </summary>
		public Bitmap Image {
			get { return image; }
		}

		/// <summary>
		/// Gets the name of the file (the filename minus directory)
		/// </summary>
		public string Name {
			get {
				FileInfo fi = new FileInfo(filename);
				return fi.Name.Substring(0, fi.Name.LastIndexOf('.'));
			}
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="filename">File to load</param>
		public ScsFile(string filename) {
			this.filename = filename;
		}

		/// <summary>
		/// Loads the minimap and attempts to save it.
		/// </summary>
		/// <param name="pngpath">File to save the image to</param>
		/// <returns>True on success, false on failure</returns>
		public bool SaveImage(string pngpath) {
			try {
				ProcessFile();
				if (image == null) {
					return false;
				}

				// Try to save it
				image.Save(pngpath, ImageFormat.Png);
				return true;

			} catch (IOException) {
				errorMessage = "Unable to write the PNG image";
				return false;
			} catch (Exception) {
				errorMessage = "An unknown error occurred";
				return false;
			}
		}

		public void ProcessFile() {
			try {
				int dot = filename.LastIndexOf('.');
				string tgapath = filename.Substring(0, dot) + "MiniMapBG.tga";
				string ddspath = Path.ChangeExtension(filename, ".tga");
				string ddspath2 = filename.Substring(0, dot) + "MiniMap.tga";

				// First try to combine the two images
				image = ReadImage(tgapath, ddspath, ddspath2);

				if (image == null) {
					// Try it in another way
					image = ReadImageFromScsFile();
				}
			} catch (Exception) {
				errorMessage = "An unknown error occurred";
			}
		}

		private Bitmap ReadImage(string tgapath, string ddspath, string ddspath2) {
			Bitmap image, image2;

			// Terrain file (TGA)
			if (!File.Exists(tgapath)) {
				this.errorMessage = "Background TGA file not found";
				return null;
			}
			TgaFile tga = new TgaFile(tgapath);
			image = tga.GetImage();
			if (image == null) {
				this.errorMessage = tga.ErrorMessage;
				return null;
			}

			// Buildings file (DDS)
			// Check whether to use ddspath (SCS) or ddspath2 (SCS:D)
			if (!File.Exists(ddspath)) {
				if (!File.Exists(ddspath2)) {
					this.errorMessage = "Minimap TGA/DDS file not found";
					return null;
				} else {
					ddspath = ddspath2;
				}
			}
			DdsFile dds = new DdsFile(ddspath);
			image2 = dds.GetImage();
			if (image2 == null) {
				this.errorMessage = dds.ErrorMessage;
				return null;
			}

			// Combine the two
			Graphics g = Graphics.FromImage(image);
			g.DrawImageUnscaled(image2, 0, 0);

			return image;
		}

		private Bitmap ReadImageFromScsFile() {
			string stderr = "The file you selected doesn't seem to be a SCS file";
			try {
				Bitmap terrain = null, roads, ambients, buildings;
				Decompressor dc = new Decompressor(filename);
				byte[] bytes = dc.Decompress();
				/*
				// DEBUG: Save uncompressed data
				FileStream fs = new FileStream(filename + ".hex", FileMode.Create, FileAccess.Write);
				fs.Write(bytes, 0, bytes.Length);
				fs.Close();
				*/
				Input input = new Input(bytes);
				// Pattern = "RoadManager SaveLoad Success"
				byte[] pattern =
					{0x52, 0x6f, 0x61, 0x64, 0x4d, 0x61, 0x6e, 0x61,
					 0x67, 0x65, 0x72, 0x20, 0x53, 0x61, 0x76, 0x65,
					 0x4c, 0x6f, 0x61, 0x64, 0x20, 0x53, 0x75, 0x63,
					 0x63, 0x65, 0x73, 0x73};
				if (input.FindPattern(pattern)) {
					int position = input.Pos();
					TerrainReader tr = new TerrainReader(input);
					terrain = tr.GetTerrain();
					if (terrain == null) {
						errorMessage = stderr + " (terrain not found)";
						return null;
					}
					input.Seek(position - pattern.Length - 8);
					RoadReader rr = new RoadReader(input, terrain.Width);
					roads = rr.GetRoads();

					ObjectReader or = new ObjectReader(input, terrain.Width);
					ambients = or.GetAmbients();
					buildings = or.GetBuildings();

					Graphics g = Graphics.FromImage(terrain);
					if (ambients != null)  g.DrawImageUnscaled(ambients, 0, 0);
					if (roads != null)     g.DrawImageUnscaled(roads, 0, 0);
					if (buildings != null) g.DrawImageUnscaled(buildings, 0, 0);
					// Flip the image: everything is upside down
					terrain.RotateFlip(RotateFlipType.RotateNoneFlipY);
				}
				return terrain;
			} catch (Exception) {
				errorMessage = stderr + ".";
				return null;
			}
		}

	}
}
